//----------------------------------------------------------------
//
// OpenDCC
//
// Copyright (c) 2006 Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//-----------------------------------------------------------------
//
// file:      s88.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// history:   2006-05-24 V0.01 started
//            2006-10-20 V0.02 added calls for intellibox
//            2006-12-04 V0.03 typedf fr init
//            2008-07-29 V0.04 new interfaces
//
//-----------------------------------------------------------------
//
// purpose:   lowcost central station for dcc
// content:   routines for S88 query
//-----------------------------------------------------------------


#ifndef __S88_H__
#define __S88_H__


// Interface for OpenDCC-Parser

typedef enum {KEEP_SIZES, READ_FROM_EEPROM}  t_s88_init_mode;
#define S88_HW_CHANGE      0 // Bit 0: hardware event (there could be a change)
#define S88_HW_CHANGE_X    1 // Bit 0: hardware event - mirror for xpressnet
#define S88_TRNT_CHANGE    2 // Bit 0: turnout feedback event
#define S88_TRNT_CHANGE_X  3 // Bit 0: turnout feedback event - mirror for xpressnet
extern unsigned char s88_event;         // Message flag: there was a change on s88

// return true: okay
// false: size too large
unsigned char init_s88(t_s88_init_mode s88_init_mode);

void run_s88(void);                     // must be called in loop - result is s88_event 

extern unsigned char s88_size1;			// # of bytes for s88 in string 1
extern unsigned char s88_size2;
extern unsigned char s88_size3;
extern unsigned char s88_total;         // # of bytes for s88 (1 modul = 2 byte = 16 bits)



// Turnout Feedback

void enter_feedback_to_s88(unsigned int turnout_addr, unsigned char state);


// Lenz-Mode

void create_xpressnet_feedback(unsigned int index, unsigned char *message);       
void create_xpressnet_schaltinfo(unsigned int trnt_addr, unsigned char *message);
bool make_s88_lenz_feedback_report(unsigned char *message);    // this clears S88_HW_CHANGE
bool make_s88_lenz_schalt_report(unsigned char *message);      // this clears S88_TRNT_CHANGE
void make_s88_lenz_report(void);                               // call this and get the whole report

// Intellibox-Mode

void sense_off_s88(void);               // clear any pending change flag
bool make_s88_ib_report(void);
void make_s88_ib_single_report(unsigned char modul);

extern unsigned char s88_ib_report[3];

// Interface fr HSI88-Mode, Standalone

void HSI88_parser(void);

#endif



