//----------------------------------------------------------------
//
// OpenDCC
//
// Copyright (c) 2006 Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//-----------------------------------------------------------------
//
// file:      status.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// history:   2006-04-13 V0.1 started
//            2007-10-15 V0.2 key_go_at_start_pressed neu dazu.
//
//-----------------------------------------------------------------
//
// purpose:   lowcost central station for dcc
// content:   control of LED
//            timertick
//            check for key and output short
//
// 2do:       
//-----------------------------------------------------------------
#ifndef __STATUS_H__
#define __STATUS_H__

void init_tick(void);          // start up isr and timer0

void stop_tick(void);

volatile extern signed int timerval;   // timertick, increments every 5ms, wrap around

typedef struct       // each member will be decremented on every tick until 0
  {
    unsigned int rs232;        // comm timeout 
    unsigned int key_go;       // debouce GO tracer
    unsigned int key_stop;     // debouce STOP tracer
    unsigned int parser;       // high level parser timeout
    unsigned int main_short;   // debounce short message
    unsigned int prog_short;  
  } t_no_timeout;

extern t_no_timeout no_timeout;

typedef struct
  {
    unsigned char rs232_rest;     // Zeit bis zum wechsel in Ticks (5ms)
    unsigned char rs232_ontime;   // Einschaltzeit
    unsigned char rs232_offtime;  // Ausschaltzeit
    unsigned char go_rest;        // Zeit bis zum wechsel in Ticks (5ms)
    unsigned char go_ontime;      // Einschaltzeit
    unsigned char go_offtime;     // Ausschaltzeit
    unsigned char stop_rest;      // Zeit bis zum wechsel in Ticks (5ms)
    unsigned char stop_ontime;    // Einschaltzeit
    unsigned char stop_offtime;   // Ausschaltzeit
    unsigned char ctrl_rest;      // Zeit bis zum wechsel in Ticks (5ms)
    unsigned char ctrl_ontime;    // Einschaltzeit
    unsigned char ctrl_offtime;   // Ausschaltzeit
  } t_led_pwm;

extern t_led_pwm led_pwm;

extern t_opendcc_state opendcc_state;            // this is the current state of the box

//===================================================================================
// Interface-Routines to other programs
// Alle Zeiten in us

void set_led_rs232(t_rs232_led_state next);
//        case DISCONNECTED:
//        case CONNECTED:
//        case RUNNING:
//        case STOPPED:

void set_led_state(t_opendcc_state next);
//        case RUN_OKAY:
//        case RUN_STOP:             // DCC Running, all Engines Emergency Stop
//        case RUN_OFF:              // Output disabled (2*Taste, PC)
//        case RUN_SHORT:            // Kurzschlu
//        case RUN_PAUSE:            // DCC Running, all Engines Speed 0
//        case PROG_OKAY:
//        case PROG_SHORT:           //
//        case PROG_OFF:
//        case PROG_ERROR:

void test_blink_leds(void);          // running light

void error_blink_leds(void);         // blink all fast!

void set_opendcc_state(t_opendcc_state next);

void run_state(void);

unsigned char is_prog_state(void);


//===================================================================================
// Tastatur
// Hier nur bestehend aus 2 Tasten STOP und GO
//

#define UNPRESSED 0
#define DEBOUNCE  1
#define PRESSED   2

#define GO_STROKE    1
#define STOP_STROKE  2

extern unsigned char key_go_at_start_pressed;

unsigned char run_key(void);

void set_led_state(t_opendcc_state next);


void run_state_test(void) ;

#if (WATCH_S88_SIZE1 == 1) 
   void set_alarm_led(unsigned char next);
#endif

extern unsigned char ext_stop_enabled;

#endif // __STATUS_H__
